#include "cpufreq.h"

/*
 * Originally written by Peter Holik
 */

#if COLLECT_CPUFREQ
#define MODULE_NAME "cpufreq"

#include "plugin.h"
#include "common.h"

static int num_cpu=0;

static char *cpufreq_file = "cpufreq-%s.rrd";

static char *ds_def[] =
{
	"DS:value:GAUGE:25:0:U",
	NULL
};
static int ds_num = 1;

extern time_t curtime;

#define BUFSIZE 256

void cpufreq_init (void)
{
        int status;
	char filename[BUFSIZE];

	while (1)
	{
		status = snprintf (filename, BUFSIZE, "/sys/devices/system/cpu/cpu%d/cpufreq/cpuinfo_cur_freq", num_cpu);
    		if (status < 1 || status >= BUFSIZE)
			break;

		if (access(filename, R_OK))
			break;

		num_cpu++;
	}
	syslog (LOG_INFO, MODULE_NAME" found %d cpu(s)", num_cpu);
}

void cpufreq_write (char *host, char *inst, char *val)
{
        int status;
        char file[BUFSIZE];

        status = snprintf (file, BUFSIZE, cpufreq_file, inst);
        if (status < 1 || status >= BUFSIZE)
                return;

	rrd_update_file (host, file, val, ds_def, ds_num);
}

void cpufreq_submit (int cpu_num, unsigned long long val)
{
	char buf[BUFSIZE];
	char cpu[16];

	if (snprintf (buf, BUFSIZE, "%u:%llu", (unsigned int) curtime, val) >= BUFSIZE)
		return;
        snprintf (cpu, 16, "%i", cpu_num);

	plugin_submit (MODULE_NAME, cpu, buf);
}

void cpufreq_read (void)
{
        int status;
	unsigned long long val;
	int cpu_num=0;
	FILE *fp;
	char filename[BUFSIZE];
	char buffer[16];

	for (cpu_num=0; cpu_num < num_cpu; cpu_num++)
	{
		status = snprintf (filename, BUFSIZE, "/sys/devices/system/cpu/cpu%d/cpufreq/cpuinfo_cur_freq", cpu_num);
    		if (status < 1 || status >= BUFSIZE)
			return;

		if ((fp = fopen (filename, "r")) == NULL)
		{
			syslog (LOG_WARNING, "cpufreq: fopen: %s", strerror (errno));
			return;
		}

		if (fgets (buffer, 16, fp) == NULL)
		{
			syslog (LOG_WARNING, "cpufreq: fgets: %s", strerror (errno));
			return;
		}

		if (fclose (fp))
			syslog (LOG_WARNING, "cpufreq: fclose: %s", strerror (errno));

		val = atoll (buffer);

		cpufreq_submit (cpu_num, val);
	}
}
#undef BUFSIZE

void module_register (void)
{
	plugin_register (MODULE_NAME, cpufreq_init, cpufreq_read, cpufreq_write);
}

#undef MODULE_NAME
#endif /* COLLECT_CPUFREQ */
